/************************************************************************/
/************************************************************************/
/*									*/
/*                        RADICALLY PUBLIC DOMAIN                       */
/*									*/
/************************************************************************/
/* Author : Don Conover							*/
/************************************************************************/
/*									*/
/* These routines are for the FOSSIL driver program interface		*/
/* or similar drivers in the public domain.				*/
/*									*/
/* Turbo 'C' supports inline code and other extensions to the 'C'	*/
/* language that enables the writing of very small code.  This is	*/
/* not meant to be compatible with the MSC compiler.			*/
/*									*/
/************************************************************************/
/* If you need to recompile this module, you will need Turbo C 1.5 and	*/
/* MASM 4.0 or greater. Isaacson's A86 will NOT work in spite of his	*/
/* assurances to the contrary.						*/
/************************************************************************/
/*									*/
/* Additions and updates by: William Beebe				*/
/*									*/
/*                        RADICALLY PUBLIC DOMAIN                       */
/*									*/
/************************************************************************/
/************************************************************************/

#pragma		inline

#include	<dos.h>
#include	<conio.h>
#include	<stdio.h>
#include	<stdarg.h>
#include	<ctype.h>
#include	"c:\csrc\yabbs\ys\yabbs.h"
#include	"yamchek.h"
#include	"yamlib.h"

char	pbuf[256];

word	COMMPORT  = 1,		/* com port defaults to com1:		*/
	clock_ticks,		/* number of interrupt ticks/second.	*/
	tick_time,		/* timer decremented each tick time.	*/
	tseconds,		/* number of tick seconds on line.	*/
	tminutes,		/* number of tick minutes on line.	*/
	thours,			/* number of tick hours on line.	*/
	inactive,		/* # of seconds to wait without input.	*/
	time_left = 15,		/* caller's time left in minutes.	*/
	pause,			/* tick downcounter used for pauses.	*/
	pending   = 2,		/* event is pending and message sent.	*/
	chat_mode = no,		/* determines if we're already in chat.	*/
	snoop     = no,		/* echo caller's screen to local screen.*/
	both_kbd  = no,		/* accept input from remote and local.	*/
	local_kbd = yes;	/* local keyboard, no = FOSSIL.		*/

struct	_fossil	fossil;		/* declare fossil info struct.		*/

/************************************************************************/
/*									*/
/************************************************************************/
/*

local_keybd_chk() handles  inputs  from the local keyboard, specifically
from the sysop when a caller is on-line.

This routine allows  control  of  the  system  when  a  caller is on the
system.  Specifically, the  sysop  can  chat with the caller, change the
caller's priv level, raise or  lower  the time on line, enter data along
with the caller, snoop on the  caller's  activities, and kick the caller
off the system.

*/

void pascal	local_keybd_chk( void )

{
int	c,
	c2,
	d;

char	chat_start[] = "\r\nStart Chat.\r\n",
	chat_end[]   = "\r\nEnd Chat.\r\n",
	*two_minute_warnings[] =
		{
		"\r\nYou have no time left. Exiting. \r\n\b",
		"\r\nYou have one minute left.\r\n\b",
		"\r\nYou have two minutes left.\r\n\b"
		};



if (pending == time_left)	/* start warnings at 2 minutes left.	*/
	{
	--pending;
	rawdisplay(two_minute_warnings[time_left]);
	}

if (!time_left)		/* no time left, so kick ass off the system.	*/
	{
	status = 1;
	longjmp(mark,-1);
	}

if (chat_mode)
	return;

_AH = 0x0d;
geninterrupt(0x14);
if (_AX == -1)
	return;

_AH = 0x0e;
geninterrupt(0x14);

switch(toupper(_AL))
	{
	case 24:	/* ^X twit kill switch.				*/
	time_left = 0;
	break;

	case '<':
	if (time_left)
	printf("ONE MINUTE LESS (%d)\n",--time_left);
	break;

	case '>':
	printf("ONE MINUTE ADDED (%d)\n",++time_left);
	if (time_left > 2)
		pending = 2;
	break;

	case 'A':
	both_kbd = yes;
	puts("\nSimultanious keyboards ON.");
	break;

	case 'C':		/* Chat mode. Talk to caller.		*/
	d = snoop;		/* save current snoop mode.		*/
	chat_mode = snoop = yes; /* force snoop mode on, for full echo.	*/
	rawdisplay(chat_start);	/* alert caller and sysop to status.	*/
	do
		{
		_AH = 0x0d;
		geninterrupt(0x14);	/* test for keyboard input.	*/
		if (_AX != -1)		/* if local data, then...	*/
			{
			_AH = 0x0e;
			geninterrupt(0x14);	/* get the local data..	*/
			c = _AL;		/* save it for later...	*/
			if (c != ESC)		/* and send it to all.	*/
				sendbyte((c == '\r' ? '\n' : c));
			}
		
		_AH = 0x03;		/* test for remote data...	*/
		_DX = COMMPORT-1;
		geninterrupt(0x14);
		if (!(_AL & 0x80))	/* while waiting watch carrier.	*/
			{
			status = 1;
			longjmp(mark,-1);
			}
		if(_AH & 0x01)		/* if data is recieved...	*/
			{
			_AH = 0x02;
			_DX = COMMPORT-1;
			geninterrupt(0x14);	/* get remote data...	*/
			c2 = _AL;		/* and send it to all.	*/
			sendbyte((c2 == '\r' ? '\n' : c2));
			}
		}
	while (c != ESC);	/* continue until local keyboard <esc>.	*/
	rawdisplay(chat_end);	/* alert all that chat is ending.	*/
	snoop = d;		/* restore snoop mode to what it was.	*/
	chat_mode = no;		/* restore procedure to check commands.	*/
	break;

	case 'N':	/* turn snoop mode on.				*/
	snoop = yes;
	puts("\nSnoop mode on");
	break;

	case 'O':	/* turn snoop mode off.				*/
	snoop = no;
	puts("\nSnoop mode off");
	break;

	default:
	printf("\n%s, %s, Baud = %d\nTime left: %d min\n",user.name,user.city,user.flag,time_left);
	puts("C)hat");
	puts("Hangup...... ^X sudden (now)");
	puts("Snoop mode.. N)on   O)ff            Local Kbd.... A) on    <esc> off");
	puts("Time........ > more < less");
	}
}

/************************************************************************/

int cdecl setparms(int baud, int parity, int wordwidth, int stopbits)
{
register setup_mask;

switch(baud/300)
	{
	case 1:   setup_mask = 0x40;	break;
	case 2:   setup_mask = 0x60;	break;
	case 4:   setup_mask = 0x80;	break;
	case 8:   setup_mask = 0xA0;	break;
	case 16:  setup_mask = 0xC0;	break;
	case 32:  setup_mask = 0xE0;	break;
	case 64:  setup_mask = 0;	break;
	default: return(0);
	}

switch (parity)
	{
	case 0: break;
	case 1: setup_mask |= 0x8; break;
	case 2: setup_mask |= 0xA; break;
	default: return(0);
	}

switch (stopbits-1)
	{
	case 0: break;
	case 1: setup_mask |= 0x4; break;
	default: return(0);
	}

switch (wordwidth-7)
	{
	case 0: setup_mask |= 0x2; break;
	case 1: setup_mask |= 0x3; break;
	default: return(0);
	}

/* setup_mask &= 0xff;   zero's out ah */

_AX = setup_mask & 0xff;
_DX = COMMPORT-1;
geninterrupt(0x14);
return ( 1 );
}

/************************************************************************

Sendbyte.

The sendbyte function sends a byte to  whatever stream is selected.  The
global flag local_kbd must be set by the  calling  routine  in  order to
determine whether the data is read from the standard  DOS  stream or the
FOSSIL.   The FOSSIL side of the sendbyte function tests  for  linefeed.
If a linefeed is passed to it, then a carriage return  is sent after the
linefeed.

NOTE:	All  character I/O routines  that  read  the  FOSSIL  check  for
	carrier detect and will exit with the appropriate exit code (dos
	errorlevel) when carrier is dropped (the other end hangs up).

*/

void  pascal sendbyte( byte character )
{
if (local_kbd)			/* if local keyboard, then use		*/
	{			/* standard library character		*/
	if (character == '\n')
		{
		++ln_cnt;
		_AH = 0x06;
		_DL = '\r';
		asm	int	21H
		}

	_AH = 0x06;
	_DL = character;
	asm	int	21H
	return;
	}

do
	{
	local_keybd_chk();
	_AH = 0x03;		/* check driver port status.		*/
	_DX = COMMPORT-1;
	geninterrupt(0x14);
	if (!(_AL & 0x80))	/* check if carrier goes away.		*/
		{
		status = 1;
		longjmp(mark,-1);
		}
	}
while(!(_AH & 0x40));		/* wait for output buffer to EMPTY.	*/

if (character == '\n')		/* if the character is 'C' linefeed,	*/
	{			/* then send a carriage return after it.*/
	++ln_cnt;
	if (snoop)
		{
		_AH = 0x13;
		_AL = '\r';
		geninterrupt(0x14);
		}
	_AH = 0x01;
	_AL = '\r';
	_DX = COMMPORT-1;
	geninterrupt(0x14);
	}

if (snoop)
	{
	_AH = 0x13;
	_AL = character;
	geninterrupt(0x14);
	}

_AH = 0x01;			/* now send the character out the port.	*/
_AL = character;
_DX = COMMPORT-1;
geninterrupt(0x14);

}

/************************************************************************

Getbyte.

The getbyte function gets a byte  from whatever stream is selected.  The
global flag local_kbd must be set by  the  calling  routine  in order to
determine whether the data is read from the  standard  DOS stream or the
FOSSIL.

NOTE:	All  character  I/O  routines  that  read  the FOSSIL check  for
	carrier detect and will exit with the appropriate exit code (dos
	errorlevel) when carrier is dropped (the other end hangs up).

*/

byte pascal getbyte( void )
{
if (local_kbd)
	return(getch());

do
	{
	if (both_kbd)	/* if simultanious keyboards...			*/
		{
		_AH = 0x0d;
		geninterrupt(0x14);	/* test for keyboard input.	*/
		if (_AX != -1)		/* if local data, then...	*/
			{
			_AH = 0x0e;
			geninterrupt(0x14);	/* get the local data..	*/
			if (_AL != ESC)
				return(_AL);	/* and return it.	*/
			both_kbd = no;
			puts("\nSimultanious keyboards OFF.");
			}
		}
	else
	local_keybd_chk();	/* check for sysop commands		*/

	_AH = 0x03;
	_DX = COMMPORT-1;
	geninterrupt(0x14);	/* test for many items on remote input.	*/
	if (!(_AL & 0x80))		/* while waiting watch carrier.	*/
		{
		status = 1;
		longjmp(mark,-1);
		}
	}
while(!(_AH & 0x01));	/* while no data is ready on the remote end.	*/

_AH = 0x02;
_DX = COMMPORT-1;
geninterrupt(0x14);	/* now get the character waiting for us.	*/
return(_AL);
}

/************************************************************************/

void  pascal rawdisplay( register byte *string )
{
if (local_kbd)		/* fastest way to send characters to stdout.	*/
	{
	while ( *string )	/* while there are characters in the string...	*/
		{
		if (*string == '\n')
			++ln_cnt;
		_AH = 0x06;
		_DL = *string++;
		asm	int	21H
		}
	return;
	}

do
	{
	local_keybd_chk();
	_AH = 0x03;		/* check driver port status.		*/
	_DX = COMMPORT-1;
	geninterrupt(0x14);
	if (!(_AL & 0x80))		/* check if carrier goes away.	*/
		{
		status = 1;
		longjmp(mark,-1);
		}
	}
while(!(_AH & 0x40));		/* while output buffer is not enpty.	*/

while ( *string )	/* while there are characters in the string...	*/
	{
	if (*string == '\n')
		++ln_cnt;
	if (snoop)
		{
		_AH = 0x13;
		_AL = *string;
		geninterrupt(0x14);
		}
	_AH = 0x01;
	_AL = *string++;
	_DX = COMMPORT-1;
	geninterrupt(0x14);
	}
}

/************************************************************************/

/*

Display() sends a line of text to the console terminal.  Display() was
written to take the place of explicitly formating a string via
sprintf() and then sending the output to an output routine.  This
allows printf() style arguments and only one procedure call. 
Cputs-like display speed is maintained via a fixed pre-allocated
buffer.  Re-direction and ANSI compatibility are maintained by using
the DOS interrupt 6.

*/

void display( char *format, ... )

{
va_list	arg_ptr;
va_start (arg_ptr, format);
vsprintf(pbuf, format, arg_ptr);
va_end(arg_ptr);

rawdisplay(pbuf);
}

/************************************************************************/

byte pascal carrier( void )

{
_AH = 0x03;
_DX = COMMPORT-1;
geninterrupt(0x14);
return(_AL & 0x80);
}

/************************************************************************/

void pascal setdtr( unsigned char state )

{
_AH = 0x06;
_AL = state;
_DX = COMMPORT-1;
geninterrupt(0x14);
}

/************************************************************************/

byte pascal initcomm( void )

{
_AH = 0x04;
_DX = COMMPORT-1;
geninterrupt(0x14);
if ( _AX == 0x1954 )
	{
	_AH = 0x0f;
	_AL = 0xf9;
	_DX = COMMPORT-1;
	geninterrupt(0x14);     
	setdtr(1);
	return ( 1 );
	}
return ( 0 );
}


/************************************************************************/

void pascal uninitcomm( void )
{
_AH = 0x05;
_DX = COMMPORT-1;
geninterrupt(0x14);
setdtr(0);
}

/************************************************************************/

void pascal purge_outbuf( void )
{
if (local_kbd)
	return;

_AH = 0x09;
_DX = COMMPORT-1;
geninterrupt(0x14);
}

/************************************************************************/

void pascal purge_inbuf( void )
{
if (local_kbd)
	return;

_AH = 0x0a;
_DX = COMMPORT-1;
geninterrupt(0x14);
}

/************************************************************************/

void pascal watchdog( int state)
{
_AH = 0x14;
_AL = state;
_DX = COMMPORT - 1;
geninterrupt(0x14);
}

/************************************************************************/

void pascal tick_wait( word length )
{
asm	cli
pause = length;
asm	sti
while(pause);
}

/************************************************************************

This  function is really two procedures in one.  The latter half of  the
procedure installs INT_TICKER, an assembly  language  routine  whose far
address  is  installed  in  the  FOSSIL    timer    tick    chain.    If
invoke_tick_timer() is called with a non-zero value,  then  the  routine
INT_TICKER  is installed.  If invoke_tick_timer is called  with  a  zero
value, then INT_TICKER is removed.  In addition to  installing  the code
far address, the current data segment is saved in the routines sav_ds so
that it can be recovered and data can be manipulated in the DGROUP.  The
routine returns  a  completion  value;    -1 if installation or deletion
failed, and zero if successful.

INT_TICKER counts down the value in tick_time, and when  it goes to zero
an  hh:mm:ss  real-time  clock is updated, as well as an  inactivity  on
input  timer  and  the total time (in minutes) left to a  caller.    All
though this is a small model, INT_TICKER has a retf and the  far pointer
via the offset and the code segment are passed to the FOSSIL.

*/

int pascal invoke_tick_timer( int register state )

{
asm	jmp	short main_code
asm	sav_ds	dw 0

asm	int_ticker:
asm	pushf			/* save incoming interrupt flag state.	*/
asm	cli			/* disable interrupts.			*/
asm	mov	ax,cs:sav_ds	/* set up data segment to our data.	*/
asm	mov	ds,ax

asm	mov	ax,_pause	/* decrement the pause counter.		*/
asm	dec	ax
asm	js	int_ticker6	/* don't roll past zero.		*/
asm	mov	_pause,ax
int_ticker6:

asm	mov	ax,_tick_time	/* count tick time down.		*/
asm	dec	ax
asm	jne	int_ticker2

asm	mov	ax,_inactive	/* count down inactivity timer.		*/
asm	dec	ax
asm	js	int_ticker4	/* don't roll past zero.		*/
asm	mov	_inactive,ax	/* decrement the inactive input timer.	*/
int_ticker4:

asm	inc	_tseconds	/* increment minutes modulo 60.		*/
asm	mov	ax,_tseconds
asm	cmp	ax,60
asm	jb	int_ticker3
asm	xor	ax,ax
asm	mov	_tseconds,ax

asm	mov	ax,_time_left	/* is any time left for this call?	*/
asm	dec	ax		/* subtract a minute.			*/
asm	js	int_ticker5	/* don't roll past zero.		*/
asm	mov	_time_left,ax
int_ticker5:

asm	inc	_tminutes

int_ticker3:
asm	mov	ax,_clock_ticks	/* reload tick timer.			*/

int_ticker2:
asm	mov	_tick_time,ax	/* update tick timer.			*/
asm	popf
asm	retf

main_code:

if (state)
	{
	asm	mov	ah,7	/* get number of ticks/second.		*/
	asm	int	14h
	asm	mov	al,ah
	asm	xor	ah,ah
	asm	mov	_clock_ticks,ax
	asm	mov	_tick_time,ax

	asm	mov	ax,cs	/* get the segment and offset.		*/
	asm	mov	es,ax
	asm	mov	dx, offset int_ticker
	asm	mov	sav_ds,ds		/* save current DS.	*/
	asm	mov	ax,1601h	/* try to install INT_TICKER.	*/
	asm	int	14h
	return(_AX);		/* return completion status.		*/
	}

asm	mov	ax,cs
asm	mov	es,ax
asm	mov	dx, offset int_ticker
asm	mov	ax,1600h		/* de-install INT_TICKER.	*/
asm	int	14h
return(_AX);
}

/************************************************************************

Get Fossil Info, Function 27 (1BH).

Gets information about the FOSSIL in use.  Requires prior declaration of
a fossil structure.  Returns pointer to structure of type _fossil.

NOTE:	According  to my experiences with X00 1.08, you need to INIT the
	FOSSIL before you call this function or no data will be moved to
	the data structure.  Why this is I don't know, but I will remedy
	this with the release of my FOSSIL.

*/

word pascal get_fossil_info( void )
{
asm	push	di		/* save DI to keep from clobbering TC.	*/
asm	mov	dx, _COMMPORT	/* point to comm port we're using.	*/
asm	dec	dx
asm	mov	ax,ds		/* move data segment to ES.		*/
asm	mov	es,ax
asm	mov	di, offset DGROUP:_fossil	/* load data offset.	*/
asm	mov	ax, 1b00h			/* load function.	*/
asm	mov	cx, 19		/* load size of data area.		*/
asm	int	14h
asm	pop	di		/* restore DI for TC's sake.		*/
return	(_AX);
}
